<?php
/**
 * @file
 * Contains the ElementController class.
 */

class ElementController extends EntityAPIController implements EntityAPIControllerRevisionableInterface {

  /**
   * Implements EntityAPIControllerInterface.
   *
   * @param $transaction
   *   Optionally a DatabaseTransaction object to use. Allows overrides to pass
   *   in their transaction object.
   */
  public function save($entity, DatabaseTransaction $transaction = NULL) {
    global $user;
    if (!empty($entity->is_new)) {
      // Doing this allows the author to be programtically set oncreation.
      $entity->id = NULL;
      $entity->uid = !empty($entity->uid) ? $entity->uid : $user->uid;
      $entity->created = REQUEST_TIME;
    }
    else {
      $entity->is_new_revision = TRUE;
    }
    $entity->changed = REQUEST_TIME;
    // EntityAPIController borks the saving of authors on revisions. This was a
    // much later addition to the controller by the community and does not
    // quite fully satisfy the "node-like" revisions we are trying to accomplish
    // here. For this reason, we do a complete rehash of the save logic, rather
    // than call parent::save(). Although the code is almost identical, we need
    // to add one line right in the middle, setting entity->uid to the current
    // user before a revision is saved.
    $transaction = isset($transaction) ? $transaction : db_transaction();
    try {
      // Load the stored entity, if any.
      if (!empty($entity->{$this->idKey}) && !isset($entity->original)) {
        // In order to properly work in case of name changes, load the original
        // entity using the id key if it is available.
        $entity->original = entity_load_unchanged($this->entityType, $entity->{$this->idKey});
      }
      $entity->is_new = !empty($entity->is_new) || empty($entity->{$this->idKey});
      $this->invoke('presave', $entity);

      if ($entity->is_new) {
        $return = drupal_write_record($this->entityInfo['base table'], $entity);
        if ($this->revisionKey) {
          $this->saveRevision($entity);
        }
        $this->invoke('insert', $entity);
      }
      else {
        // Update the base table if the entity doesn't have revisions or
        // we are updating the default revision.
        if (!$this->revisionKey || !empty($entity->{$this->defaultRevisionKey})) {
          $return = drupal_write_record($this->entityInfo['base table'], $entity, $this->idKey);
        }
        if ($this->revisionKey) {
          // Update the UID of the entity so that the revision table recieves
          // the UID of the user doing the updating. If this is being done
          // programtically, i.e. $user->uid = 0, then use the original author
          // as the revisioner.
          $entity->uid = $user->uid;
          $return = $this->saveRevision($entity);
        }
        $this->resetCache(array($entity->{$this->idKey}));
        $this->invoke('update', $entity);
        // Field API always saves as default revision, so if the revision saved
        // is not default we have to restore the field values of the default
        // revision now by invoking field_attach_update() once again.
        if ($this->revisionKey && !$entity->{$this->defaultRevisionKey} && !empty($this->entityInfo['fieldable'])) {
          field_attach_update($this->entityType, $entity->original);
        }
      }
      // Ignore slave server temporarily.
      db_ignore_slave();
      unset($entity->is_new);
      unset($entity->is_new_revision);
      unset($entity->original);

      return $return;
    }
    catch (Exception $e) {
      $transaction->rollback();
      watchdog_exception($this->entityType, $e);
      throw $e;
    }
  }

}
